* filename: i2c.asm *
* -contains I2C timer interrupt service and other I2C initialization *
*  routines *


		.mmregs          	   
      	.include "time.inc"
      	.include "i2c.inc"
  		.include "bsp.inc"  
  		.include "i2creg.inc"      
  		
  		.def TIMER_INT
  		.def _I2CSTATUS,_ODBYTECTR,_IDBYTECTR,_ODPTR,_SLAVE_ADDR            
      	.def _init_gpio,_init_i2c,_write_i2c,_read_i2c
      	.ref _USER_FUNCTION
      	 
      	 ;***MASK array defines the bit position of a byte
		.sect "constant"
MASK    .int 01h			;Bit 0 
        .int 02h            ;Bit 1
        .int 04h            ;Bit 2
        .int 08h            ;Bit 3
        .int 10h            ;Bit 4
        .int 20h            ;Bit 5
        .int 40h            ;Bit 6 
        .int 80h            ;Bit 7 (MSB)
		
		.sect "program"				
;***************************************************************************
; TIMER ISR
;***************************************************************************
TIMER_INT:								

;***SAVE CPU REGISTERS TO STACK***
	  NOP
	  NOP
	  PSHM	ST0
	  PSHM	ST1
	  PSHM	AR1
	  PSHM	AR2
	  PSHM	AR3
	  PSHM  AR4
	  PSHM  AR5
	  PSHM	AR6
	  NOP
	  NOP                  
	  RSBX	SXM                 		;Suppressed sign extension
	  RSBX	CPL                         ;Use DP as relative direct-addressing
	  RSBX	HM                          ;Hold  mode = 0
      LD	#_I2C_DP,DP      			;Set DP  
      NOP
      NOP
      LD	_I2CSTATUS,A	    		;load i2c status
      LD	A,B							;Duplicate i2cstatus
  	  AND   #K_DISABLE_I2C,B            ;Check if i2c disable
      BC    tint_disable_tint,BNEQ		;If =1, goto end
;***UPDATE PREVIOUS SDA AND SCL***      
      AND   #3H,A	 					;mask all except current bits
      LD    A,2,B 						;shift current bits to previous bits
      ANDM  #0FFF0H,_I2CSTATUS       	;mask off lower 4 bits
      LD	_I2CSTATUS,A	    		;load i2c status
      ADD	B,A							;current -> previous	
      STL   A,_I2CSTATUS				;store back to register(current=0)
;***UPDATE CURRENT SDA AND SCL***
	  STM   #SPSA,AR2                	;AR2 points to SPSA
	  ST	#PCR_SUB,*AR2+          	;Check line status of SCL
  	  LD	#(K_DR_STAT_1|K_FSXP_1),B 	;Mask DR_STAT & FSXP bit (bit 4 & 3) 
  	  AND   #0FFFFH,A                 	;Clear AH(i2c status)
   	  AND	*AR2-,B						;check DR_STAT & FSXP bit
  	  OR    B,-3,A						;shift current SCL to bit 1 & 0
  	  STL   A,_I2CSTATUS             	;store back to i2c status register
;***CHECK START/STOP CONDITION OR MODE OF OPERATION***
  	  LD	A,B							;Duplicate i2cstatus
  	  AND   #K_CLKH_1,B					;Need to set SCL=high
  	  BC    tint_clk_go_high,BNEQ		;If one, goto set_high
  	  LD	A,B							;Duplicate i2cstatus
  	  AND   #K_CLKL_1,B					;Need to set SCL=low
  	  BC    tint_clk_go_low,BNEQ		;If one, goto set_low        
  	  LD	A,B							;Duplicate i2cstatus
  	  AND   #K_ACK_1,B 		   			;Check ACK in i2c status
  	  BC	tint_ack,BNEQ				;If ACK=1, goto tinit_ack
  	  LD	A,B							;Duplicate i2cstatus
  	  AND   #K_START_1,B    			;Check start condition in i2c status
  	  BC	tint_start,BNEQ				;If start=1, goto tinit_start
  	  AND   #K_STOP_1,A   	 			;Check stop condition in i2c status
  	  BC	tint_stop,ANEQ				;If stop=1, goto tinit_stop
  	  MVDK	_I2CSTATUS,TEMP_REG			;Copy i2c status to temp register
	  ANDM	#01100000B, TEMP_REG	 	;Mask on 6 and 5 bit (Mode of operation)
	  CMPM	TEMP_REG,#K_READ_FROM_SLAVE	;Compare only 6 and 5 bit
	  BC	tint_read_slave, TC         ;If 10,goto read_slave(TEMP_REG as an
	  									;extra ACK flag,not equal to 0)      
  	  CMPM  TEMP_REG,#K_WRITE_TO_SLAVE  ;Compare only 6 and 5 bit     
  	  BC	tint_write_slave, TC        ;If 01,goto write slave(TEMP_REG
  	  									;as a extra ACK flag,not equal to 0)      
  	  B		tint_busy					;Multi-master not supported now
;***END OF MAIN CONTROL LOOP***

;**************************************************************************
;**************************************************************************    

;*****START CONDITION SEQUENCES*****
tint_start:  	                                                   
	  ;***READ THE PREVIOUS AND CURRENT SDA/SCL LEVELS***
	  MVDK	_I2CSTATUS,TEMP_REG			;Copy i2c status to temp register
	  ANDM	#0FH, TEMP_REG	 			;Mask on lower 4 bits
	  CMPM	TEMP_REG,#1111B			    ;All lines high
	  BC	tint_start_start,TC         ;Goto end of start condition
	  CMPM	TEMP_REG,#1110B				;Start finished
	                 ;1~~~              ;Previous SCL=1
	                 ;~1~~              ;Previous SDA=1
	                 ;~~1~              ;Current SCL=1
	                 ;~~~0              ;Current SDA=0
	  BC	tint_start_finish,TC        ;Goto end of start condition
	  B	    tint_start_invalid  		;Goto invalid (only consider 2 cases)
;***BEGIN OF START CONDITION***      
tint_start_start:
      CALL  pullSDA						;SDA->0 while SCL=1
      ;***SETUP TIMER INTERRUPT FOR hold time=5us
      STM	#STOP_TIMER,TCR     		;stop timer
      STM	#K_START_HOLD_TIME,PRD    	;load hold time of CLK (5us)
      STM	#START_TIMER,TCR    		;start timer
      B		tint_end_isr
;***END OF START CONDITION***
tint_start_finish: 
	  CALL	pullSCL						;SCL=0 after hold time 
	  ANDM	#K_START_0,_I2CSTATUS		;Clear start flag
	  ST	#1,TEMP_REG					;Set extra ACK flag for normal clocking   
      B		tint_end_isr				;TINT still at 5us
;***UNKNOWN STATE***
tint_start_invalid:
      CALL	releaseSDA					;Release both lines and do start again
      CALL	releaseSCL                  ;                               
      B		tint_end_isr			    ;End timer ISR


;**************************************************************************
;**************************************************************************    


;******BEGIN ACKNOWLEDGEMENT BIT SEQUENCE*****      
tint_ack:
      ;***CHECK IF MASTER-RECEIVER MODE (read slave)
	  MVDK	_I2CSTATUS,TEMP_REG			;Copy i2c status to temp register
	  ANDM	#01100000B, TEMP_REG	 	;Mask on 6 and 5 bit (Mode of operation)
	  CMPM	TEMP_REG,#K_READ_FROM_SLAVE	;Compare only 6 and 5 bit
	  BC	tint_ack_not_rs, NTC        ;If not 10,goto not_rs

      LD    _I2CSTATUS,A                ;Check if sending of slave addr is over?
      AND	#K_ADDR_1,A                 ;
      BC	tint_ack_end_slave_addr,ANEQ;if end of send addr, goto end_slave_addr

	  LD	POINTER,B					;Extra flag
	  BC	tint_ack_rs_first,BEQ		;If first time,POINTER==0		
	  CALL	releaseSDA
	  ANDM	#K_ACK_0,_I2CSTATUS			;Remove ACK flag      
	  B		tint_end_isr	
tint_ack_rs_first:	  	  
	  ST	#1,POINTER      			;Set POINTER flag     
	  LD	_I2CSTATUS,A				;Load i2c status 
	  AND   #K_STOP_1,A   	 			;Check stop condition in i2c status
  	  BC	tint_ack_stop,ANEQ			;If stop=1, goto tint_ack_stop
  	  CALL	pullSDA						;ACK is LOW when not last byte
  	  B		tint_ack_end                ;TEMP_REG flag is not 0
tint_ack_stop:
	  CALL	releaseSDA					;ACK is HIGH when last byte 
	  B		tint_ack_end                ;TEMP_REG flag is not 0	  
  	  
tint_ack_end_slave_addr:
      ANDM	#K_ADDR_0,_I2CSTATUS		;Remove slave address flag		 
                                        ;continue on, let slave ACK
  	  ;***IN MASTER-TRANSMIT MODE (write slave)***									
tint_ack_not_rs:	  
	  CALL	releaseSDA					;SDA=high,let slave ACK on this line                                                 
      ST	#0,TEMP_REG					;Clear temp reg,as an extra ACK flag	  
tint_ack_end:
      ORM	#K_CLKH_1,_I2CSTATUS		;Next TINT,enable SCL -> high                                                                       	  
      B		tint_end_isr	
;***TEST IF SLAVE ACKNOWLEDGE TO MASTER***
tint_ack_test:                          ;Call from where SCL=high(CLKH flag removed)            
      ;***CHECK IF SLAVE ACK BY PULLING SDA DOWN***
	  ;ST    SPSA,AR2                	;AR2 points to SPSA(redundant,safety)
	  ST	#PCR_SUB,*AR2+              ;Sub-bank address (SPSA)=PCR
  	  LD	#K_FSXP_1,A   	            ;Check if SDA go low?
      AND	*AR2-,A    
      BC	tint_ack_sda_low,AEQ        ;if SDA=low,goto sda_low
      ;***SLAVE DID NOT ACKNOWLEDGE***      
      ORM	#K_ERR_1,_I2CSTATUS			;Turn error flag on 
      ST	#K_ERR_NOACK,ERRORCODE		;Give type of error
tint_ack_sda_low:      
      ;***SLAVE ACKNOWLEDGED***         ;SCL go_high flag has disabled.
      ORM	#K_CLKL_1,_I2CSTATUS		;Enable SCL go_low flag  
      ANDM	#(K_ACK_0&K_ADDR_0),_I2CSTATUS;Remove ACK and ADDR flag      
      STM	#STOP_TIMER,TCR     		;stop timer
      STM	#K_SCL_HIGH,PRD    			;load high period of SCL (at 5us)
      STM	#START_TIMER,TCR    		;start timer
	  B		tint_end_isr    	        ;Interrupt at 5us
	  

;**************************************************************************
;**************************************************************************    

      
;*****STOP CONDITION SEQUENCES*****      
tint_stop:                              ;SCL=low after ACK
	  ;***CHECK IF FIRST TIME ENTER STOP SEQUENCE***
	  LD	TEMP1_REG,A					;To make sure transition of SDA does 
	  BC	tint_stop_continue,ANEQ		;not occur in SCL=1
	  CALL  pullSDA	                    ;If extra ACT flag=1,not first time 
	  ST	#1,TEMP1_REG				;Set flag=1       
      STM	#STOP_TIMER,TCR     		;stop timer
      STM	#K_STOP_SETUP_TIME,PRD    	;load stop setup time (5us)
      STM	#START_TIMER,TCR    		;start timer
	  B		tint_end_isr	            ;check again
tint_stop_continue:	
	  MVDK	_I2CSTATUS,TEMP_REG			;Copy i2c status to temp register
	  ANDM	#03H,TEMP_REG	 			;Mask on lower 2 bits
	  ;***AFTER 2ND PASS, SCL=1 , SDA=0***  
	  CMPM	TEMP_REG,#10B				;Stop finished
	                 ;1~    	        ;Current SCL=1
	                 ;~0        	    ;Current SDA=0             
	  BC	tint_stop_finish,TC         ;Goto end of stop condition
	  ;***IMMEDIATELY AFTER ACK, SCL/SDA=0/0***
	  CMPM	TEMP_REG,#00B				;After ACK
	                 ;0~    	        ;Current SCL=0
	                 ;~0				;Current SDA=0
	  BC	tint_stop_invalid,NTC		;Goto invalid (only consider 2 cases)
	  ;***CHECK IF SLAVE HOLD ON TO SCL LINE***
      CALL  releaseSCL					;SCL=1
      NOP                 				;Wait for a while. (Does SCL go high
	  NOP                               ;Wait for a while.   immediately?)
	  NOP                               ;Wait for a while.
      ;***SYNCHRONIZATION***
	  ;ST    SPSA,AR2                	;AR2 points to SPSA(redundant,safety)
	  ST	#PCR_SUB,*AR2+              ;Sub-bank address (SPSA)=PCR
  	  LD	#K_DR_STAT_1,A              ;Check if SCL go high?
      AND	*AR2-,A    
      BC	tint_stop_scl_go_high_yes,ANEQ		;if SCL=high, goto high_yes
	  ;***SLAVE STILL HOLDING ON TO SCL LINE***
	  ;***SETUP TIMER INTERRUPT FOR 1us***
      B		tint_stop_invalid			;Check again
      
tint_stop_scl_go_high_yes:      
      ;***SETUP TIMER INTERRUPT FOR hold time=5us***
      STM	#STOP_TIMER,TCR     		;stop timer
      STM	#K_STOP_SETUP_TIME,PRD    	;load stop setup time (5us)
      STM	#START_TIMER,TCR    		;start timer
      B		tint_end_isr
	  ;***SDA=0 -> 1***
tint_stop_finish: 
	  CALL	releaseSDA					;SDA=1
	  ANDM	#K_STOP_0,_I2CSTATUS		;Clear stop flag
      ORM	#K_SUCC_1,_I2CSTATUS        ;SUCCESSFUL
      NOP
      CALL  _USER_FUNCTION				;Goto user-function subroutine
      NOP 
      B		tint_end_isr				;End timer ISR
tint_stop_invalid:
      CALL	pullSDA   					;Enable the situation of stop condition
      CALL  releaseSCL                  ;=>Both lines are low                               
	  STM	#STOP_TIMER,TCR     		;stop timer
      STM	#K_PRD_VAL,PRD    			;load PRD (1us)
      STM	#START_TIMER,TCR    		;start timer
      B		tint_end_isr	            ;check again    
      
      
;**************************************************************************
;**************************************************************************      
      

;*****READ FROM SLAVE SEQUENCES*****      
tint_read_slave:                                             
	  LD	_I2CSTATUS,A	    		;load i2c status
  	  AND   #K_ADDR_1,A					;Check if send slave addr now
      BC    tint_clk_go_high,AEQ		;If =0, goto rs_mode
      B		tint_write_slave			;Write slave addr in the first byte
;***READ DATA FROM SLAVE DURING SCL=HIGH***      
tint_rs_mode:                                                      
	  ;***SETUP POINTERS TO CORRECT LOCATION***
	  LD    #IDPTR,B					;Load addr of out-data to B
      LD    _IDBYTECTR, -8,A			;Move current byte pointer
      STL	A,POINTER					;Store it to POINTER (1ST= BYTE 0)
      ADD   A,B							;Update current btye pointer
      STLM  B,AR4						;AR4 points to current byte
      
      CMPM	IDBITCTR,#7                 ;If IDBITCTR == 7, then it is new byte
      BC	tint_rs_not_new_byte,NTC	;else not new byte
      ST	#0,*AR4						;Clear the memory location
tint_rs_not_new_byte:      
	  ST	#PCR_SUB,*AR2+          	;Check current line status of SDA
  	  LD	#K_FSXP_1,B 				;Mask FSXP bit (bit 3) 
  	  AND	*AR2-,B						;check FSXP bit
      LD	B,-3,A	  					;Store to bit 0 of A
      
      LD	IDBITCTR,B   				;Check if end of byte,ie. bit ctr=0
      BC	tint_rs_bit_0,BEQ			;If bit ctr=0, goto new_byte
      SUB   #1,B                        ;else decrement bit ctr
      STL	B,TEMP1_REG                 ;Store to TEMP1 register

      RPT	TEMP1_REG                   ;Shift A (bitctr-1) times
      SFTL	A,1                         ;Result in A         
      ;***LSB NO NEED TO SHIFT***
tint_rs_bit_0:                          ;If bit 0, no need to shift
      LD	*AR4,B                      ;Load B with current read byte
      OR	A,B                         ;Combine and store to B
      AND   #0FFH,B                     ;Allows only byte
      STL	B,*AR4                      ;Store back to memory

      LD	IDBITCTR,A	                ;If BITCTR=0(LSB) ie. end of byte
      BC    tint_rs_next_byte,AEQ		;goto next byte
      SUB	#1,A,B						;Decrement A and store to B
      STL	B,IDBITCTR					;Store back to register
	  B	    tint_rs_next            	;Next bit
	  ;***TEST IF COME TO LAST BYTE***
tint_rs_next_byte:
	  ORM	#K_ACK_1,_I2CSTATUS			;Enable ACK flag
      ST	#7,IDBITCTR					;Start from MSB (initiate next byte)
      ADDM	#1,POINTER					;Increment POINTER
      ANDM	#0FFH,_IDBYTECTR			;Prepare num of btye to receive

	  LD	POINTER,B					;Load the incremental POINTER to B
      LD    _IDBYTECTR,A                ;Load num of byte to A
      SUB   B,A							;A=(Num of byte) - (current byte POINTER)     
      BC	tint_rs_no_more_byte,AEQ	;If=0,no more to receive
      LD	POINTER,8,A                 ;Shift 8 places forward
      LD	_IDBYTECTR,B				;
      OR	B,A							;Combine POINTER and num of btye to send                                                                 
      STL	A,_IDBYTECTR				;Update IDBYTECTR
      B		tint_rs_next				;Goto next byte
tint_rs_no_more_byte:
      ORM	#K_STOP_1,_I2CSTATUS		;Enable STOP condition
tint_rs_next:
	  ST	#0,POINTER					;Extra flag use in ack_rs
	  ORM	#K_CLKL_1,_I2CSTATUS		;Next TINT,enable releaseSCL	
      STM	#STOP_TIMER,TCR     		;stop timer
      STM	#K_SCL_HIGH,PRD    			;load PRD (interrupt at 5us)
      STM	#START_TIMER,TCR    		;start timer
      B		tint_end_isr				;

      
;**************************************************************************
;**************************************************************************      
 
      
;*****WRITE TO SLAVE SEQUENCES*****
tint_write_slave:                       
	  ;***SET UP AR4 POINTS TO BYTE TO BE SENT***
      LD    #_ODPTR,B					;Load addr of out-data to B
      LD    _ODBYTECTR, -8,A			;Move current byte pointer
      STL	A,POINTER					;Store it to POINTER (1ST= BYTE 0)
      ADD   A,B							;Update current btye pointer
      STLM  B,AR4						;AR4 points to current byte
      ;***SET UP AR3 POINTS TO BIT TO BE SENT***
      STM	#MASK,AR3					;AR3 points to MASK
	  RPT	ODBITCTR					;Look up table (LUT)
	  LD	*AR3+,A						;Store mask to A  
	  AND	*AR4, A						;Check if the bit is high or low
	  ;***WRITE BIT***
	  CC	releaseSDA,ANEQ             ;Bit=1, SDA=high
	  CC    pullSDA,AEQ                 ;Bit=0, SDA=low
      ;***TEST IF COME TO LAST BIT***
      LD	ODBITCTR,A	                ;If BITCTR=0(LSB) ie. end of byte
      BC    tint_ws_next_byte,AEQ		;goto next byte
      SUB	#1,A,B						;Decrement A and store to B
      STL	B,ODBITCTR					;Store back to register
	  B	    tint_ws_next            	;Next bit
	  ;***TEST IF COME TO LAST BYTE***
tint_ws_next_byte:
	  LD	POINTER,B                   ;First byte is byte zero
	  ORM	#K_ACK_1,_I2CSTATUS			;Enable ACK flag
      ST	#7,ODBITCTR					;Start from MSB (initiate next byte)
      ADD	#1,B						;Increment POINTER
      STL   B,POINTER					;Store back
      ANDM	#0FFH,_ODBYTECTR			;Prepare num of btye to send
      LD    _ODBYTECTR,A                ;Load num of byte to A
      SUB   B,A							;A=(Num of byte) - (current byte POINTER)     
      BC	tint_ws_end_write,AEQ		;If=0,no more to send,enable stop condition
      LD	POINTER,8,A                 ;Shift 8 places forward
      LD	_ODBYTECTR,B				;
      OR	B,A							;Combine POINTER and num of btye to send                                                                 
      STL	A,_ODBYTECTR				;Update ODBYTECTR
	  B		tint_ws_next				;Goto next byte
      ;***NO MORE DATA***
tint_ws_end_write:                      ;Do not update POINTER for last byte
	  ORM	#K_STOP_1,_I2CSTATUS		;Enable STOP condition 
	  ST	#0,TEMP1_REG
      ;***PREPARE FOR NEXT BIT***
tint_ws_next:
	  ORM	#K_CLKH_1,_I2CSTATUS		;Next TINT,enable releaseSCL
  	  STM	#STOP_TIMER,TCR     		;stop timer
      STM	#K_SCL_LOW1,PRD    			;load SCL low period(interrupt at 5us)
      STM	#START_TIMER,TCR    		;start timer     
      B		tint_end_isr				;
      ;***BUSY LINES OR UNKNOWN CONDITION
tint_busy:                      		;Unknown condition or lines are busy
      ;***REPORT ERROR TYPE***
      ORM	#K_ERR_1,_I2CSTATUS			;Turn error flag on 
      ST	#K_ERR_UNKNOWN,ERRORCODE	;Give type of error
      B		tint_end_isr


;**************************************************************************
;**************************************************************************      


;*****MAKE CLK->HIGH SEQUENCES*****
tint_clk_go_high:
	  CALL  releaseSCL					;SCL=high
	  NOP                 				;Wait for a while. (Does SCL go high
	  NOP                               ;Wait for a while.   immediately?)
	  NOP                               ;Wait for a while.
      ;***SYNCHRONIZATION***
	  ;ST    SPSA,AR2                	;AR2 points to SPSA(redundant,safety)
	  ST	#PCR_SUB,*AR2+              ;Sub-bank address (SPSA)=PCR
  	  LD	#K_DR_STAT_1,A              ;Check if SCL go high?
      AND	*AR2-,A    
      BC	tint_clk_go_high_yes, ANEQ  ;if SCL=high, goto high_yes
	  ;***SLAVE HOLD SCL LOW***      
      STM	#STOP_TIMER,TCR     		;stop timer
      STM	#K_PRD_VAL,PRD    			;load PRD (interrupt at 1us)
      STM	#START_TIMER,TCR    		;start timer
      B		tint_end_isr        		;Do go_high procedure again

;***CLK GONE HIGH (SLAVE NO MORE HOLDING ONTO SCL)***
tint_clk_go_high_yes:
	  ANDM	#K_CLKH_0,_I2CSTATUS		;Remove SCL go_high flag            						

      ;*****ACKNOWLEDGE SEQUENCE***      
	  ;***CHECK IF IT IS IN THE ACK PHASE (write slave)***
	  LD	TEMP_REG,A					;Check if Slave release SDA line
	  BC	tint_ack_test,AEQ			;Extra ACT flag=0 : ACK in progress
      ;*****END OF ACKNOWLEDGE SEQUENCE***

      ;*****READ THE CURRENT SDA LEVEL (FOR READ SLAVE MODE)***
      ;***CHECK IF MASTER-RECEIVER MODE (read slave)***
	  LD	_I2CSTATUS,B				    ;
	  LD	B,A							;Duplicate  
	  AND   #K_ADDR_1,A 	   			;Check ADDR in i2c status
  	  BC	tint_clk_go_high_not_rs,ANEQ;If sending slave addr,goto not_rs
	  
	  MVDK	_I2CSTATUS,TEMP_REG			;Copy i2c status to temp register
	  ANDM	#01100000B, TEMP_REG	 	;Mask on 6 and 5 bit (Mode of operation)
	  CMPM	TEMP_REG,#K_READ_FROM_SLAVE	;Compare only 6 and 5 bit
	  BC	tint_clk_go_high_not_rs,NTC ;If not 10,goto not_rs
	  ;***CHECK IF IN ACK PHASE***
      AND   #K_ACK_1,B 		   			;Check ACK in i2c status
  	  BC	tint_rs_mode,BEQ			;If ACK=0, goto rs_mode
      ;***CHECK IF GOING TO STOP SEQUENCE***
  	  LD	_I2CSTATUS,A				;Check if come to the last byte
  	  AND	#K_STOP_1,A					;Need to set extra TEMP_REG flag=0
  	  BC	tint_clk_go_high_not_rs,AEQ ;Bcos to avoid the unknown transition of SDA
  	  ST	#0,TEMP1_REG				;To prolong SDA=0 in stop sequences									
  	  ;*****END OF READ THE CURRENT SDA LEVEL (FOR READ SLAVE MODE)***

tint_clk_go_high_not_rs:      
      ;***END OF READ SLAVE MODE***        	  
      ORM	#K_CLKL_1,_I2CSTATUS		;Enable SCL go_low flag
      STM	#STOP_TIMER,TCR     		;stop timer
      STM	#K_SCL_HIGH,PRD    			;load PRD (interrupt at 5us)
      STM	#START_TIMER,TCR    		;start timer
	  B		tint_end_isr    	        ;Interrupt at 5us


;**************************************************************************
;**************************************************************************      


;***MAKE CLK->LOW SEQUENCES***
tint_clk_go_low: 
	  CALL	pullSCL						;SCL=low               
	  ANDM	#K_CLKL_0,_I2CSTATUS		;Remove SCL go_low flag
	  STM	#STOP_TIMER,TCR     		;stop timer
      STM	#K_SCL_LOW2,PRD    			;load SCL low period(interrupt at 5us)
      STM	#START_TIMER,TCR    		;start timer     
      B		tint_end_isr	        	;Interrupt at 5us


;**************************************************************************
;**************************************************************************      

      
	  ;***DISABLE FURTHER TIMER INTERRUPT***		  
tint_disable_tint: 	  
	  ANDM	#0FFF7H,IMR					;Disable any further TINT
	  B     tint_end_end
	  
;******END OF ISR*****
tint_end_isr:
   	  ORM	#0008H,IMR         			;enable timer interrupt
 	  STM	#0FFFFH,IFR         		;clear any pending interrupts 	       
;***START OF OTHER TINT ISR
tint_end_end:
      ;***RESTORE CPU REGISTERS
      NOP
	  NOP
	  POPM	AR6
	  POPM	AR5
	  POPM	AR4
	  POPM	AR3
	  POPM	AR2
	  POPM	AR1
	  POPM	ST1
	  POPM	ST0
	  NOP
	  NOP
                                        ;END OF TIMER ISR
      RETE                              ;enable global INTM                    
      
;**************************************************************************
;**************************************************************************      


      
;***
;OTHER SUBROUTINES
;***        




;**************************************************************************
; SDA = HIGH ISR
;**************************************************************************
releaseSDA:  
	ST	#PCR_SUB,*AR2+                  ;Sub-bank address (SPSA0)=PCR0 
	ORM #K_DX_STAT_1, *AR2-			    ;Mask on DX_STAT BIT   
	NOP
	NOP
	RET
;**************************************************************************
; SDA = LOW ISR
;**************************************************************************
pullSDA: 
	ST	#PCR_SUB,*AR2+                  ;Sub-bank address (SPSA0)=PCR0 
    ANDM #K_DX_STAT_0,*AR2-             ;Mask off DX_STAT BIT
    NOP
    NOP
    RET            
;**************************************************************************
; SCL = HIGH ISR
;**************************************************************************
releaseSCL:  
	ST	#PCR_SUB,*AR2+                  ;Sub-bank address (SPSA0)=PCR0 
	ORM #K_FSRP_1,*AR2-			        ;Mask on FSR BIT   
    NOP
    NOP
	RET
;**************************************************************************
; SCL = LOW ISR
;**************************************************************************
pullSCL: 
	ST	#PCR_SUB,*AR2+                  ;Sub-bank address (SPSA0)=PCR0 
    ANDM #K_FSRP_0,*AR2-                ;Mask off FSR BIT
    NOP
    NOP
    RET                 
;**************************************************************************
;**************************************************************************

;**************************************************************************    
; SET MCBSP AS GPIO
;************************************************************************** 
_init_gpio:
	 NOP
	 PSHM  AR2
	 NOP
	 STM   #SPSA,AR2                    ;AR2 points to SPSA0

     ST    #SPCR2_SUB,*AR2+             ;Sub-bank address (SPSA0)=SPCR20
     ST    #0000000000000000b,*AR2-     ;
           ;~~~~~~~~~~~~~~~0 (XRST)     ;GPIO : transmitter is reset
      
     ST    #SPCR1_SUB,*AR2+             ;Sub-bank address (SPSA0)=SPCR20
     ST    #0000000000000000b,*AR2-     ;
           ;~~~~~~~~~~~~~~~0 (RRST)     ;GPIO : receiver set
    
     ST	  #PCR_SUB,*AR2+                ;Sub-bank address (SPSA0)=PCR0 
	 ST    #0011010100100101b,*AR2-     ;
           ;~~1~~~~~~~~~~~~~ (XIOEN)	;GPIO transmitter is enabled
           ;~~~1~~~~~~~~~~~~ (RIOEN)	;GPIO receiver is enabled
           ;~~~~0~~~~~~~~~~~ (FSXM)     ;FSX set as input 
           ;~~~~~1~~~~~~~~~~ (FSRM)     ;FSR set as output  
    	   ;~~~~~~~1~~~~~~~~ (CLKRM)	;CLKR set as output
    	   ;~~~~~~~~~~1~~~~~ (DX_STAT)  ;Release SDA line
    	   ;~~~~~~~~~~~~~1~~ (FSRP)	    ;Release SCL line
    	   ;~~~~~~~~~~~~~~~1 (CLKRP)    ;CLKR = 1
    
      NOP                               ;McBSP takes 2 clock cycle 
      NOP                               ;to start
      POPM	AR2
      NOP
      RET
;*************************************************************************
;*************************************************************************    


;*************************************************************************    
; INITIATE I2C
;*************************************************************************    
_init_i2c:      
	  NOP
	  PSHM	AR1
	  NOP
	  STM	#_I2CSTATUS,AR1
	  ST    #(K_RESET_I2C|K_CSDA_1|K_CSCL_1),*AR1+  ;I2CSTATUS
	  ST	#0,*AR1+   								;ODBYTECTR
	  ST	#7,*AR1+								;ODBITCTR
	  ST	#0,*AR1+   								;IDBYTECTR
	  ST	#7,*AR1+								;IDBITCTR
	  ST	#0,*AR1+   								;ERRORCODE  
	  ST	#K_SLAVEADDR,*AR1						;SLAVE_ADDR
      NOP
;************************************************************************    
; INIT TIMER  
;************************************************************************
      STM	#STOP_TIMER,TCR     ;stop timer
      STM	#K_PRD_VAL,PRD      ;load PRD
      STM	#START_TIMER,TCR    ;start timer
	  ORM	#0008H,*(IMR)       ;enable timer interrupt
   	  STM	#0FFFFH,IFR         ;clear any pending interrupts
      NOP
      POPM	AR1
      NOP
      RET
;***********************************************************************	
;***********************************************************************			

;***********************************************************************    
; INIT MASTER-TRANSMITTER PROCESS  
;***********************************************************************
_write_i2c:
	  NOP
	  PSHM	AR1
	  PSHM	AR2
	  NOP
	  STM	#_I2CSTATUS,AR1     ;Init i2cstatus register for write 
	  ST	#(K_START_1|K_WRITE_TO_SLAVE|K_ADDR_1|K_CSDA_1|K_CSCL_1),*AR1
      NOP
      STM	#_SLAVE_ADDR,AR1    ;1ST loc of ODPTR is slave address
      LD	*AR1,1,A            ;Shift slave address 1 place to the left
      ADD	#K_I2C_WRITE,A      ;Write mode of operation to LSB
      STM	#_ODPTR,AR2
      STL	A,*AR2              ;Store back to _ODPTR
      STM	#_ODBYTECTR,AR1
      ADDM	#1,*AR1             ;Add one more byte for slave address
      NOP
      POPM	AR2
      POPM	AR1
	  NOP      
	  RETE		
	  
;************************************************************************	
;************************************************************************			

;************************************************************************    
; INIT MASTER-RECEIVER PROCESS  
;************************************************************************
_read_i2c:
	  NOP      
	  PSHM	AR1
	  NOP
	  STM	#_I2CSTATUS,AR1     ;Init i2cstatus register for read 
	  ST	#(K_START_1|K_READ_FROM_SLAVE|K_ADDR_1|K_CSDA_1|K_CSCL_1),*AR1
      NOP
      STM	#_SLAVE_ADDR,AR1    ;1ST loc of ODPTR is slave address
      LD	*AR1,1,A            ;Shift slave address 1 place to the left
      ADD	#K_I2C_READ,A		;Write mode of operation to LSB
      STM	#_ODPTR,AR1         
      STL	A,*AR1              ;Store back to _ODPTR
      NOP
      STM	#_ODBYTECTR,AR1     ;Only 1 byte(slave addr) to be send
      ST	#2,*AR1				;Must store 1+1 num of bytes
                                
      NOP
      POPM	AR1
	  NOP
      RETE
	  
;************************************************************************    
;**************END*OF*FILE***********************************************    
;************************************************************************    

	.end
    
    